@@ -7,7 +7,7 @@ use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(spawn receive msg snd wt snd_wt listener open_node);
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use Carp;
 use EV;
@@ -36,6 +36,9 @@ my @rcv    = ();
 my %r_bufs = ();
 my %w_bufs = ();
 
+my %pack   = ();
+my %unpack = ();
+
 my %fh2rw = ();
 my %fh2ww = ();
 
@@ -83,18 +86,21 @@ sub snd_wt($$;@) {
 }
 
 
-sub listener($$) {
-	my ($host, $port) = @_;
+sub listener($$;%) {
+	my ($host, $port, %args) = @_;
 	defined $host or carp("Argument host required"), return;
 	defined $port or carp("Argument port required"), return;
 	my $sock = IO::Socket::INET->new(Proto => 'tcp', Blocking => 0, LocalHost => $host, LocalPort => $port, Listen => 20, ReuseAddr => 1);
 	if ($sock) {
+		_pack_unpack($sock, %args) or return;
 		$listener{$sock} = $sock;
 		$fh2rw{$sock} = EV::io($sock, EV::READ, sub {
 			my $w = shift;
 			my $fh = $w->fh;
 			$DEBUG > 1 and print "Read event for listener from $self_vpid: \n";
 			my $sock = $fh->accept;
+			$pack{$sock}   = $pack{$fh};
+			$unpack{$sock} = $unpack{$fh};
 			$sock->sockopt(SO_KEEPALIVE, 1);
 			my $vpid = refaddr $sock;
 			$node{$sock}     = $vpid;
@@ -111,8 +117,8 @@ sub listener($$) {
 }
 
 
-sub open_node($$) {
-	my ($host, $port) = @_;
+sub open_node($$;%) {
+	my ($host, $port, %args) = @_;
 	defined $host or carp("Argument host required"), return;
 	defined $port or carp("Argument port required"), return;
 	my $sock = IO::Socket::INET->new(Proto => 'tcp', Blocking => 0);
@@ -120,6 +126,7 @@ sub open_node($$) {
 	$sock->sockopt(SO_KEEPALIVE, 1);
     my $rv = $sock->connect($addr);
 	if ($rv) {
+		_pack_unpack($sock, %args) or return;
 		my $vpid = refaddr $sock;
 		$node{$sock}     = $vpid;
 		$fh2vpid{$sock}  = $vpid;
@@ -134,6 +141,32 @@ sub open_node($$) {
 }
 
 
+sub _pack_unpack($%) {
+	my ($fh, %args) = @_;
+	if (my $pack = $args{pack} and my $unpack = $args{unpack}) {
+		my $r = eval {
+			my $r = $unpack->($pack->({a => ["b"]}));
+			if (ref $r eq "HASH" and ref $$r{a} eq "ARRAY" and
+				$$r{a}[0] and $$r{a}[0] eq "b")
+			{
+				return 1;
+			} else {
+				return 0;
+			}
+		};
+		if (not $r or $@) {
+			carp "False pack unpack test";
+			return;
+		}
+		$pack{$fh}   = $pack;
+		$unpack{$fh} = $unpack;
+	} elsif ($args{pack} or $args{unpack}) {
+		carp "pack and unpack is pair options";
+		return;
+	}
+	return 1;
+}
+
 
 sub receive(&) {
 	my ($receive) = @_;
@@ -173,6 +206,9 @@ sub receive(&) {
 			%r_bufs = ();
 			%w_bufs = ();
 
+			%pack   = ();
+			%unpack = ();
+
 			%fh2rw = ();
 			%fh2ww = (); 
 
@@ -261,7 +297,12 @@ sub w_event_cb_reg {
 					}
 				}
 				unless (exists $w_bufs{$fh}) {
-					my $packet = freeze shift @{$snd{$to}};
+					my $packet;
+					if (my $pack = $pack{$fh}) {
+						$packet = $pack->(shift @{$snd{$to}});
+					} else {
+						$packet = freeze  shift @{$snd{$to}};
+					}
 					my $buf = join "", pack("N", length $packet), $packet;
 					$w_bufs{$fh} = $buf;
 					$DEBUG and (@{$snd{$to}} or delete $snd{$to});
@@ -292,7 +333,12 @@ sub r_event_cb {
 							$r_bufs{$fh} = $buf || "";
 							$DEBUG and ($r_bufs{$fh} or delete $r_bufs{$fh});
 
-							my ($from, $to, $msg, $args) = @{thaw $packet};
+							my ($from, $to, $msg, $args);
+							if (my $unpack = $unpack{$fh}) {
+								($from, $to, $msg, $args) = @{$unpack->($packet)};
+							} else {
+								($from, $to, $msg, $args) = @{thaw $packet};
+							}
 
 							if ($node{$fh}) {
 								$from = $node{$fh};
@@ -333,10 +379,12 @@ sub r_event_cb {
 				delete $fh2fh{$fh};
 				delete $vpid2fh{$fh2vpid{$fh}};
 				delete $fh2vpid{$fh};
+				delete $pack{$fh};
+				delete $unpack{$fh};
 				if (my $vpid = $node{$fh}) {
 					delete $node{$fh};
 					if ($msg{NODE_CLOSED}) {
-						$msg{NODE_CLOSED}->($vpid);
+						$msg{NODE_CLOSED}->($vpid, $fh->connected ? 1 : 0);
 						w_event_cb_reg();
 					}
 				}
@@ -366,7 +414,7 @@ sub r_event_cb {
 			unless (exists $vpid2fh{$waited_vpid}) {
 				EV::unloop();
 				return;
-			}			
+			}
 		} else {
 			while (my $rcv = shift @rcv) {
 				my ($from, $msg, $args)= @{$rcv};
